﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客（YCCustomer）に付随するデータリストの親クラス
    /// </summary>
    public abstract class YCCustomerSubList : YCDbList
    {
        private YCCustomer _parent = null;

        protected YCCustomerSubList(YCConnection connection)
            : base(connection)
        {
        }

        /// <summary>
        /// リソースを解放します。
        /// </summary>
        public override void Dispose()
        {
            if (this.IsDisposable)
            {
                _parent = null;
            }

            base.Dispose();
        }

        /// <summary>
        /// 保存を許可します。
        /// </summary>
        public override bool AllowSave
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// Insert,Update,Delete文を自動生成しません。
        /// </summary>
        protected override bool AutoSQL
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// 保存の前処理
        /// </summary>
        /// <returns>キャンセルならfalse</returns>
        protected override bool PreSave()
        {
            this.SetTrans();    //トランザクションをデータベースオブジェクトに設定

            return base.PreSave();
        }

        /// <summary>
        /// 従属する顧客オブジェクトを取得、設定します。
        /// </summary>
        public YCCustomer Parent
        {
            get
            {
                return _parent;
            }
            set
            {
                if (_parent != null) throw new YCException("顧客オブジェクトは既に設定されています。");
                
                _parent = value;
            }
        }

        /// <summary>
        /// 従属する顧客オブジェクトの顧客コードを取得します。
        /// </summary>
        protected string CustomerCode
        {
            get
            {
                if (this.Parent == null) throw new YCException("顧客オブジェクトが設定されていません。");
                if (this.Parent.IsNew) throw new YCException("新規顧客オブジェクトが確定されてません。");

                return this.Parent.KeyCode;
            }
        }

        /// <summary>
        /// データコンテナの示すデータを追加します。
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public virtual void Add(YCDataContainer container)
        {
            throw new YCException("このメソッドは実装されていません。");
        }

        /// <summary>
        /// 簡易なリスト追加です。
        /// <para>各リストに主要データのみで追加します。</para>
        /// </summary>
        /// <param name="data">データ</param>
        public virtual void Add(string data)
        {
            throw new YCException("このメソッドは実装されていません。");
        }

        /// <summary>
        /// データコンテナの示すデータを変更します。
        /// </summary>
        /// <param name="container">データコンテナ</param>
        public virtual void Update(YCDataContainer container)
        {
            throw new YCException("このメソッドは実装されていません。");
        }

        /// <summary>
        /// 簡易なリスト変更です。
        /// <para>各リストに主要データのみ変更します。</para>
        /// </summary>
        /// <param name="code">主キーコード</param>
        /// <param name="data">データ</param>
        public virtual void Update(Guid code, string data)
        {
            throw new YCException("このメソッドは実装されていません。");
        }

        /// <summary>
        /// データを削除します。
        /// </summary>
        /// <param name="code">主キーコード</param>
        public virtual void Delete(Guid code)
        {
            throw new YCException("このメソッドは実装されていません。");
        }

    }
}
