﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using YSystem;

namespace YMyCustomer
{
    /// <summary>
    /// 顧客グループ選択用ダイアログ
    /// </summary>
    public partial class dlgCustomerGroupTree : YCDialog
    {
        private YCCustomerGroupNodeItem _root = null;
        private string _selectedCustomerGroupCode = string.Empty;
        private string _selectedCustomerGroupName = string.Empty;
        private bool _allowSelectNothing = false;

        public dlgCustomerGroupTree()
        {
            InitializeComponent();
        }

        /// <summary>
        /// リソースを解放します。
        /// </summary>
        private void DisposeForm()
        {
            if(!this.IsDisposed)
            {
                if(_root != null)
                {
                    try { _root.Dispose(); }
                    catch { }
                    _root = null;
                }
            }
        }

        /// <summary>
        /// ルート顧客グループ
        /// </summary>
        private YCCustomerGroupNodeItem Root
        {
            get
            {
                if (_root == null)
                {
                    _root = new YCCustomerGroupNodeItem(YCConnectionManager.Instance.Connection);
                    _root.CustomerGroupCode = string.Empty;
                    _root.CustomerGroupName = "顧客グループ";
                }

                return _root;
            }
        }

        /// <summary>
        /// 選択された顧客グループコード
        /// </summary>
        public string SelectedCustomerGroupCode
        {
            get
            {
                return _selectedCustomerGroupCode;
            }
            set
            {
                _selectedCustomerGroupCode = value;
            }
        }

        /// <summary>
        /// 選択された顧客グループ名
        /// </summary>
        public string SelectedCustomerGroupName
        {
            get
            {
                return _selectedCustomerGroupName;
            }
            set
            {
                _selectedCustomerGroupName = value;
            }
        }

        /// <summary>
        /// 選択なしを許可するかどうかを取得、設定
        /// <para>デフォルトはfalse</para>
        /// </summary>
        public bool AllowSelectNothing
        {
            get
            {
                return _allowSelectNothing;
            }
            set
            {
                _allowSelectNothing = value;
            }
        }

        /// <summary>
        /// TreeViewの初期化
        /// </summary>
        private void InitializeTreeView()
        {
            TreeNode node = null;

            try
            {
                tvCustomerGroup.ImageList = new ImageList();
                tvCustomerGroup.ImageList.Images.Add("Close", YMyCustomer.Properties.Resources.CloseFolder);
                tvCustomerGroup.ImageList.Images.Add("Open", YMyCustomer.Properties.Resources.OpenFolder);

                node = this.CreateTreeNode(this.Root);

                tvCustomerGroup.Nodes.Add(node);
                this.LoadChildrenNode(node);

                node = null;
            }
            catch (Exception e)
            {
                node = null;
                throw e;
            }
        }

        /// <summary>
        /// ノードの生成
        /// </summary>
        /// <param name="item">顧客グループアイテム</param>
        /// <returns>ノード</returns>
        private TreeNode CreateTreeNode(YCCustomerGroupNodeItem item)
        {
            TreeNode node = null;

            try
            {
                node = new TreeNode();
                node.Tag = item;
                node.Text = ((YCCustomerGroupNodeItem)node.Tag).CustomerGroupName;
                node.ImageKey = "Close";
                node.SelectedImageKey = "Close";

                return node;
            }
            catch (Exception e)
            {
                node = null;
                throw e;
            }
        }

        /// <summary>
        /// 子ノードの読込み
        /// </summary>
        /// <param name="target">対象ノード</param>
        private void LoadChildrenNode(TreeNode target)
        {
            if (target.Tag == null) throw new YCException("顧客グループアイテムが不正です。");

            YCCustomerGroupNodeItem item = null;
            TreeNode node = null;

            try
            {
                item = ((YCCustomerGroupNodeItem)target.Tag);

                if (!item.IsLoaded)
                {
                    item.Fill();
                    item.MoveFirst();

                    while (!item.EOF)
                    {
                        node = this.CreateTreeNode(item.CreateChildNodeItem());

                        target.Nodes.Add(node);
                        node = null;

                        item.MoveNext();
                    }
                }
            }
            catch (Exception e)
            {
                item = null;
                node = null;
                throw e;
            }

        }

        /// <summary>
        /// ノードが開かれる時の処理
        /// <para>ノードを先読みします。</para>
        /// </summary>
        /// <param name="node">対象ノード</param>
        private void ExpandNode(TreeNode node)
        {
            foreach (TreeNode childNode in node.Nodes)
            {
                this.LoadChildrenNode(childNode);
            }
        }

        //イベント
        private void dlgCustomerGroupTree_Load(object sender, EventArgs e)
        {
            try
            {
                this.InitializeTreeView();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void tvCustomerGroup_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            try
            {
                this.ExpandNode(e.Node);
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.tvCustomerGroup.SelectedNode == null
                    || ((YCCustomerGroupNodeItem)this.tvCustomerGroup.SelectedNode.Tag).IsRoot)
                {
                    if (this.AllowSelectNothing)
                    {
                        this.SelectedCustomerGroupCode = string.Empty;
                        this.SelectedCustomerGroupName = string.Empty;
                    }
                    else
                    {
                        throw new YCException("顧客グループが選択されていません。");
                    }
                }
                else
                {
                    this.SelectedCustomerGroupCode = ((YCCustomerGroupNodeItem)this.tvCustomerGroup.SelectedNode.Tag).CustomerGroupCode;
                    this.SelectedCustomerGroupName = ((YCCustomerGroupNodeItem)this.tvCustomerGroup.SelectedNode.Tag).CustomerGroupName;
                }

                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void tvCustomerGroup_AfterExpand(object sender, TreeViewEventArgs e)
        {
            try
            {
                e.Node.ImageKey = "Open";
                e.Node.SelectedImageKey = "Open";
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }

        private void tvCustomerGroup_AfterCollapse(object sender, TreeViewEventArgs e)
        {
            try
            {
                e.Node.ImageKey = "Close";
                e.Node.SelectedImageKey = "Close";
            }
            catch (Exception err)
            {
                YCMessageBox.ShowErrorMessage(err.Message);
            }
        }
    }
}
